/**
 * Copyright (c) 2015 Ivan Bausov <ivan.bausov@gmail.com> (MIT Licensed)
 * A part of B:STRUCT package <https://github.com/ivan-bausov/bstruct>
 */
/// <reference path="./definitions/node-0.10.d.ts" />
/// <reference path="./definitions/node-0.10.d.ts" />
/// <reference path="./definitions/underscore.d.ts" />
var _ = require('underscore');
var enums = require('./compiler.e');
var StructCompiler = require('./struct_compiler.t');
var ScssCompiler = require('./scss_compiler.t');
var HtmlCompiler = require('./html_compiler.t');
var TYPES = enums.TYPES;
var Block = (function () {
    function Block(source_object) {
        this.source_object = source_object;
    }
    Block.prototype.getSCSS = function () {
        return new ScssCompiler(this.source_object).compile();
    };
    Block.prototype.getHTML = function () {
        return new HtmlCompiler(this.source_object).compile();
    };
    Block.prototype.getName = function () {
        return 'block-' + this.source_object.data.name;
    };
    return Block;
})();
var Compiler = (function () {
    function Compiler(source_code) {
        this.source_code = source_code;
        this.source_object = new StructCompiler(source_code).compile();
    }
    Compiler.prototype.getSCSS = function () {
        return new ScssCompiler(this.source_object).compile();
    };
    Compiler.prototype.getHTML = function () {
        return new HtmlCompiler(this.source_object).compile();
    };
    Compiler.prototype.getBlocks = function () {
        return _.compact(_.map(this.source_object.children, function (item) {
            if (item.data.type === TYPES.BLOCK) {
                return new Block(item);
            }
            else {
                return null;
            }
        }));
    };
    Compiler.ITEM_TYPE = TYPES;
    return Compiler;
})();
module.exports = Compiler;
