/**
 * Copyright (c) 2015 Ivan Bausov <ivan.bausov@gmail.com> (MIT Licensed)
 * A part of B:STRUCT package <https://github.com/ivan-bausov/bstruct>
 */
var _ = require('underscore');
var enums = require('./compiler.e');
var TYPES = enums.TYPES;
var Compiler = (function () {
    function Compiler(data) {
        this.scss = this.renderToSCSS(data).replace(/{BLOCK}_/g, '');
    }
    Compiler.prototype.compile = function () {
        return this.scss;
    };
    Compiler.prototype.renderToSCSS = function (item) {
        var _this = this;
        var scss = Compiler.renderSCSSTemplate(item), children_scss = [], children_scss_string;
        _.each(item.children, function (child) {
            children_scss.push(_this.renderToSCSS(child));
        });
        if (item.data) {
            if (item.data.type === TYPES.PLACEHOLDER) {
                children_scss_string = children_scss.join('\n\n');
            }
            else {
                children_scss_string = _.map(children_scss.join('\n').split('\n'), function (str) {
                    return str && ('  ' + str);
                }).join('\n');
            }
        }
        else {
            children_scss_string = children_scss.join('\n\n');
        }
        if (item.data && item.data.type === TYPES.BLOCK) {
            children_scss_string = children_scss_string.replace(/{BLOCK}/g, item.data.name);
        }
        return scss.replace('{CHILDREN}', children_scss_string);
    };
    Compiler.renderSCSSTemplate = function (item) {
        var template, name;
        if (item.data) {
            if (item.data.type === TYPES.PLACEHOLDER) {
                template = '{CHILDREN}';
            }
            else {
                template = '{name} {\n{CHILDREN}\n}';
            }
            name = Compiler.compileElementName(item);
            return template.replace('{name}', name);
        }
        else {
            return '{CHILDREN}';
        }
    };
    Compiler.compileElementName = function (item) {
        var name = '', data = item.data;
        if (data.type === TYPES.ELEMENT) {
            if (data.name) {
                name = (data.tag || '') + '.{BLOCK}_' + data.name;
            }
            else {
                name = '& > ' + (data.tag || 'div');
            }
        }
        else if (data.type === TYPES.BLOCK) {
            name = '.block-' + data.name;
        }
        return name;
    };
    return Compiler;
})();
module.exports = Compiler;
