/**
 * Copyright (c) 2015 Ivan Bausov <ivan.bausov@gmail.com> (MIT Licensed)
 * A part of B:STRUCT package <https://github.com/ivan-bausov/bstruct>
 */
var _ = require('underscore');
var Item = (function () {
    function Item(parent, data) {
        this.parent = parent;
        this.data = data;
        this.childs = [];
    }
    Item.prototype.getParent = function () {
        return this.parent;
    };
    Item.prototype.getInfo = function () {
        return this.data;
    };
    Item.prototype.getChildren = function () {
        return this.childs;
    };
    Item.prototype.addChild = function (item_data) {
        var child = new Item(this, item_data);
        this.childs.push(child);
        return child;
    };
    Item.prototype.serialize = function () {
        return {
            data: this.getInfo(),
            children: _.map(this.getChildren(), function (item) {
                return item.serialize();
            })
        };
    };
    return Item;
})();
var Tree = (function () {
    function Tree() {
        this.root = new Item(null, null);
        this.current_leaf = this.root;
    }
    Tree.prototype.add = function (leaf_data) {
        this.current_leaf = this.current_leaf.addChild(leaf_data);
    };
    Tree.prototype.get = function () {
        return this.current_leaf;
    };
    Tree.prototype.up = function () {
        this.current_leaf = this.current_leaf.getParent() || this.current_leaf;
    };
    Tree.prototype.upTo = function (level) {
        while (this.level() > level) {
            this.up();
        }
    };
    Tree.prototype.level = function () {
        var level = 0, item = this.current_leaf;
        while (item = item.getParent()) {
            level++;
        }
        return level;
    };
    Tree.prototype.serialize = function () {
        return this.root.serialize();
    };
    return Tree;
})();
module.exports = Tree;
